@@ -1,6 +1,6 @@
 /* Declarations for convert.c
-   Copyright (C) 2003, 2004, 2005, 2006, 2009, 2010, 2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2006, 2009, 2010, 2011, 2015 Free
+   Software Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -31,18 +31,20 @@ as that of the covered work.  */
 #ifndef CONVERT_H
 #define CONVERT_H
 
-struct hash_table;		/* forward decl */
+struct hash_table;              /* forward decl */
 extern struct hash_table *dl_url_file_map;
 extern struct hash_table *downloaded_html_set;
 extern struct hash_table *downloaded_css_set;
 
 enum convert_options {
-  CO_NOCONVERT = 0,		/* don't convert this URL */
-  CO_CONVERT_TO_RELATIVE,	/* convert to relative, e.g. to
+  CO_NOCONVERT = 0,             /* don't convert this URL */
+  CO_CONVERT_TO_RELATIVE,       /* convert to relative, e.g. to
                                    "../../otherdir/foo.gif" */
-  CO_CONVERT_TO_COMPLETE,	/* convert to absolute, e.g. to
-				   "http://orighost/somedir/bar.jpg". */
-  CO_NULLIFY_BASE		/* change to empty string. */
+  CO_CONVERT_BASENAME_ONLY,     /* convert the file portion only (basename)
+                                   leaving the rest of the URL unchanged */
+  CO_CONVERT_TO_COMPLETE,       /* convert to absolute, e.g. to
+                                   "http://orighost/somedir/bar.jpg". */
+  CO_NULLIFY_BASE               /* change to empty string. */
 };
 
 struct url;
@@ -51,36 +53,36 @@ struct url;
    position in an HTML document, etc.  */
 
 struct urlpos {
-  struct url *url;		/* the URL of the link, after it has
-				   been merged with the base */
-  char *local_name;		/* local file to which it was saved
-				   (used by convert_links) */
+  struct url *url;              /* the URL of the link, after it has
+                                   been merged with the base */
+  char *local_name;             /* local file to which it was saved
+                                   (used by convert_links) */
 
   /* reserved for special links such as <base href="..."> which are
      used when converting links, but ignored when downloading.  */
-  unsigned int ignore_when_downloading	:1;
+  unsigned int ignore_when_downloading  :1;
 
   /* Information about the original link: */
 
-  unsigned int link_relative_p	:1; /* the link was relative */
-  unsigned int link_complete_p	:1; /* the link was complete (had host name) */
-  unsigned int link_base_p	:1; /* the url came from <base href=...> */
-  unsigned int link_inline_p	:1; /* needed to render the page */
-  unsigned int link_css_p	:1; /* the url came from CSS */
-  unsigned int link_expect_html	:1; /* expected to contain HTML */
-  unsigned int link_expect_css	:1; /* expected to contain CSS */
+  unsigned int link_relative_p  :1; /* the link was relative */
+  unsigned int link_complete_p  :1; /* the link was complete (had host name) */
+  unsigned int link_base_p  :1;     /* the url came from <base href=...> */
+  unsigned int link_inline_p    :1; /* needed to render the page */
+  unsigned int link_css_p   :1;     /* the url came from CSS */
+  unsigned int link_expect_html :1; /* expected to contain HTML */
+  unsigned int link_expect_css  :1; /* expected to contain CSS */
 
-  unsigned int link_refresh_p	:1; /* link was received from
-				       <meta http-equiv=refresh content=...> */
-  int refresh_timeout;		/* for reconstructing the refresh. */
+  unsigned int link_refresh_p   :1; /* link was received from
+                                       <meta http-equiv=refresh content=...> */
+  int refresh_timeout;              /* for reconstructing the refresh. */
 
   /* Conversion requirements: */
-  enum convert_options convert;	/* is conversion required? */
+  enum convert_options convert;     /* is conversion required? */
 
   /* URL's position in the buffer. */
   int pos, size;
 
-  struct urlpos *next;		/* next list element */
+  struct urlpos *next;              /* next list element */
 };
 
 /* downloaded_file() takes a parameter of this type and returns this type. */
