@@ -1,7 +1,7 @@
 /* Handling of recursive HTTP retrieving.
    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation,
-   Inc.
+   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2015 Free Software
+   Foundation, Inc.
 
 This file is part of GNU Wget.
 
@@ -37,12 +37,7 @@ as that of the covered work.  */
 #include <unistd.h>
 #include <errno.h>
 #include <assert.h>
-#ifdef ENABLE_THREADS
-#include <pthread.h>
-#include <semaphore.h>
 
-#include "multi.h"
-#endif
 #include "url.h"
 #include "recur.h"
 #include "utils.h"
@@ -55,7 +50,8 @@ as that of the covered work.  */
 #include "html-url.h"
 #include "css-url.h"
 #include "spider.h"
-
+#include "exits.h"
+
 /* Functions for maintaining the URL queue.  */
 
 struct queue_element {
@@ -164,26 +160,42 @@ url_dequeue (struct url_queue *queue, struct iri **i,
   xfree (qel);
   return true;
 }
-
-static bool download_child_p (const struct urlpos *, struct url *, int,
-                              struct url *, struct hash_table *, struct iri *);
-static bool descend_redirect_p (const char *, struct url *, int,
-                                struct url *, struct hash_table *, struct iri *);
 
-#ifdef ENABLE_THREADS
-static void *
-start_retrieve_url (void *arg)
+static void blacklist_add (struct hash_table *blacklist, const char *url)
 {
-  struct s_thread_ctx *ctx = (struct s_thread_ctx *) arg;
+  char *url_unescaped = xstrdup (url);
 
-  ctx->status = retrieve_url (ctx->url_parsed, ctx->url,
-                              &ctx->file, &ctx->redirected,
-                              ctx->referer, &ctx->dt,
-                              false, ctx->i, true, NULL);
-  ctx->terminated = 1;
-  sem_post (ctx->retr_sem);
+  url_unescape (url_unescaped);
+  string_set_add (blacklist, url_unescaped);
+  xfree (url_unescaped);
 }
-#endif
+
+static int blacklist_contains (struct hash_table *blacklist, const char *url)
+{
+  char *url_unescaped = xstrdup(url);
+  int ret;
+
+  url_unescape (url_unescaped);
+  ret = string_set_contains (blacklist, url_unescaped);
+  xfree (url_unescaped);
+
+  return ret;
+}
+
+typedef enum
+{
+  WG_RR_SUCCESS, WG_RR_BLACKLIST, WG_RR_NOTHTTPS, WG_RR_NONHTTP, WG_RR_ABSOLUTE,
+  WG_RR_DOMAIN, WG_RR_PARENT, WG_RR_LIST, WG_RR_REGEX, WG_RR_RULES,
+  WG_RR_SPANNEDHOST, WG_RR_ROBOTS
+} reject_reason;
+
+static reject_reason download_child (const struct urlpos *, struct url *, int,
+                              struct url *, struct hash_table *, struct iri *);
+static reject_reason descend_redirect (const char *, struct url *, int,
+                              struct url *, struct hash_table *, struct iri *);
+static void write_reject_log_header (FILE *);
+static void write_reject_log_reason (FILE *, reject_reason,
+                              const struct url *, const struct url *);
 
 /* Retrieve a part of the web beginning with START_URL.  This used to
    be called "recursive retrieval", because the old function was
@@ -210,17 +222,6 @@ uerr_t
 retrieve_tree (struct url *start_url_parsed, struct iri *pi)
 {
   uerr_t status = RETROK;
-#ifdef ENABLE_THREADS
-  const int N_THREADS = opt.jobs > 0 ? opt.jobs : 1;
-  sem_t retr_sem;
-  int free_threads = N_THREADS;
-  char *next_url = NULL, *next_referer;
-  int next_depth;
-  bool next_html_allowed, next_css_allowed;
-  struct iri *next_i = NULL;
-
-  struct s_thread_ctx *thread_ctx;
-#endif
 
   /* The queue of URLs we need to load. */
   struct url_queue *queue;
@@ -231,6 +232,8 @@ retrieve_tree (struct url *start_url_parsed, struct iri *pi)
 
   struct iri *i = iri_new ();
 
+  FILE *rejectedlog = NULL; /* Don't write a rejected log. */
+
 #define COPYSTR(x)  (x) ? xstrdup(x) : NULL;
   /* Duplicate pi struct if not NULL */
   if (pi)
@@ -243,12 +246,6 @@ retrieve_tree (struct url *start_url_parsed, struct iri *pi)
     set_uri_encoding (i, opt.locale, true);
 #undef COPYSTR
 
-#ifdef ENABLE_THREADS
-  thread_ctx = calloc (N_THREADS, sizeof *thread_ctx);
-  sem_init (&retr_sem, 0, 0);
-  /* FIXME: CHECK FOR ERRORS.  */
-#endif
-
   queue = url_queue_new ();
   blacklist = make_string_hash_table (0);
 
@@ -256,21 +253,24 @@ retrieve_tree (struct url *start_url_parsed, struct iri *pi)
      just URL so we enqueue the canonical form of the URL.  */
   url_enqueue (queue, i, xstrdup (start_url_parsed->url), NULL, 0, true,
                false);
-  string_set_add (blacklist, start_url_parsed->url);
+  blacklist_add (blacklist, start_url_parsed->url);
+
+  if (opt.rejected_log)
+    {
+      rejectedlog = fopen (opt.rejected_log, "w");
+      write_reject_log_header (rejectedlog);
+      if (!rejectedlog)
+        logprintf (LOG_NOTQUIET, "%s: %s\n", opt.rejected_log, strerror (errno));
+    }
 
   while (1)
     {
       bool descend = false;
-      char *file = NULL;
-      bool is_css = false;
-      bool dash_p_leaf_HTML = false;
-      char *url = NULL, *referer;
+      char *url, *referer, *file = NULL;
       int depth;
       bool html_allowed, css_allowed;
-      bool dequed = false;
-#ifdef ENABLE_THREADS
-      int index = 0;
-#endif
+      bool is_css = false;
+      bool dash_p_leaf_HTML = false;
 
       if (opt.quota && total_downloaded_bytes > opt.quota)
         break;
@@ -279,231 +279,108 @@ retrieve_tree (struct url *start_url_parsed, struct iri *pi)
 
       /* Get the next URL from the queue... */
 
-#ifndef ENABLE_THREADS
       if (!url_dequeue (queue, (struct iri **) &i,
                         (const char **)&url, (const char **)&referer,
                         &depth, &html_allowed, &css_allowed))
         break;
-#else
-      if (next_url == NULL)
-        {
-          if (url_dequeue (queue, (struct iri **) &next_i,
-                           (const char **)&next_url, (const char **)&next_referer,
-                           &next_depth, &next_html_allowed, &next_css_allowed))
-            dequed = true;
-        }
-
-      i = next_i;
-      url = next_url;
-      referer = next_referer;
-      depth = next_depth;
-      html_allowed = next_html_allowed;
-      css_allowed = next_css_allowed;
-#endif
 
       /* ...and download it.  Note that this download is in most cases
-         unconditional, as download_child_p already makes sure a file
+         unconditional, as download_child already makes sure a file
          doesn't get enqueued twice -- and yet this check is here, and
-         not in download_child_p.  This is so that if you run `wget -r
+         not in download_child.  This is so that if you run `wget -r
          URL1 URL2', and a random URL is encountered once under URL1
          and again under URL2, but at a different (possibly smaller)
          depth, we want the URL's children to be taken into account
          the second time.  */
-      if (dequed && url && dl_url_file_map && hash_table_contains (dl_url_file_map, url))
+      if (dl_url_file_map && hash_table_contains (dl_url_file_map, url))
         {
-	  bool is_css_bool;
+          bool is_css_bool;
 
           file = xstrdup (hash_table_get (dl_url_file_map, url));
 
           DEBUGP (("Already downloaded \"%s\", reusing it from \"%s\".\n",
                    url, file));
 
-	  if ((is_css_bool = (css_allowed
-			      && downloaded_css_set
-			      && string_set_contains (downloaded_css_set, file)))
-	      || (html_allowed
-		  && downloaded_html_set
-		  && string_set_contains (downloaded_html_set, file)))
-	    {
-	      descend = true;
-	      is_css = is_css_bool;
-	    }
+          if ((is_css_bool = (css_allowed
+                  && downloaded_css_set
+                  && string_set_contains (downloaded_css_set, file)))
+              || (html_allowed
+                && downloaded_html_set
+                && string_set_contains (downloaded_html_set, file)))
+            {
+              descend = true;
+              is_css = is_css_bool;
+            }
         }
       else
         {
-#ifndef ENABLE_THREADS
           int dt = 0, url_err;
           char *redirected = NULL;
           struct url *url_parsed = url_parse (url, &url_err, i, true);
 
-          status = retrieve_url (url_parsed, url, &file, &redirected, referer,
-                                 &dt, false, i, true, NULL);
-#else
-          int j;
-retry:
-          if (! url)
+          if (!url_parsed)
             {
-              int used = 0;
-              for (j = 0; j < N_THREADS; j++)
-                {
-                  if (thread_ctx[j].used)
-                    {
-                      used = 1;
-                      break;
-                    }
-                }
-
-              if (! used)
-                break;
-            }
-
-          if (url && free_threads)
-            {
-              int err;
-              for (j = 0; j < N_THREADS; j++)
-                if (! thread_ctx[j].used)
-                  {
-                    index = j;
-                    free_threads--;
-                    thread_ctx[j].used = 1;
-                    thread_ctx[j].terminated = 0;
-                    break;
-                  }
-              assert (j < N_THREADS);
-
-              thread_ctx[index].file = file;
-              thread_ctx[index].referer = referer;
-              thread_ctx[index].dt = 0;
-              thread_ctx[index].i = i;
-              thread_ctx[index].redirected = NULL;
-              thread_ctx[index].range = NULL;
-              thread_ctx[index].url = url;
-              thread_ctx[index].retr_sem = &retr_sem;
-              thread_ctx[index].url_parsed = url_parse (thread_ctx[index].url,
-                                                        &thread_ctx[index].url_err,
-                                                        i, true);
-
-              err = pthread_create (&thread_ctx[index].thread, NULL,
-                                    start_retrieve_url, &thread_ctx[index]);
-              if (err == 0)
-                next_url = NULL;
-              else
-                {
-                  logprintf (LOG_NOTQUIET, "pthread_create: %s\n", strerror (errno));
-                  url_free (thread_ctx[index].url_parsed);
-                  thread_ctx[index].used = 0;
-                  free_threads++;
-                  return THREADS_ERR;
-                }
-              continue;
-            }
-
-          index = -1;
-          for (j = 0; j < N_THREADS; j++)
-            if (thread_ctx[j].used && thread_ctx[j].terminated)
-              {
-                index = j;
-                thread_ctx[j].used = 0;
-                pthread_join (thread_ctx[j].thread, NULL);
-                free_threads++;
-                break;
-              }
-
-          if (index < 0)
-            {
-              int ret;
-              do
-                ret = sem_wait (&retr_sem);
-              while (ret < 0 && errno == EINTR);
-              if (ret < 0)
-                return SEM_ERR;
-
-              goto retry;
-            }
-
-          file = thread_ctx[index].file;
-          referer = thread_ctx[index].referer;
-          i = thread_ctx[index].i;
-          url = thread_ctx[index].url;
-#endif
-
-          if (html_allowed && file && status == RETROK
-#ifndef ENABLE_THREADS
-              && (dt & RETROKF) && (dt & TEXTHTML)
-#else
-              && (thread_ctx[index].dt & RETROKF) && (thread_ctx[index].dt & TEXTHTML)
-#endif
-             )
-            {
-              descend = true;
-              is_css = false;
-            }
-
-          /* a little different, css_allowed can override content type
-             lots of web servers serve css with an incorrect content type
-          */
-          if (file && status == RETROK
-#ifndef ENABLE_THREADS
-              && (dt & RETROKF) &&
-              ((dt & TEXTCSS) || css_allowed)
-#else
-              && (thread_ctx[index].dt & RETROKF) &&
-              ((thread_ctx[index].dt & TEXTCSS) || css_allowed)
-#endif
-             )
-            {
-              descend = true;
-              is_css = true;
-            }
-
-#ifndef ENABLE_THREADS
-          if (redirected)
-#else
-          if (thread_ctx[index].redirected)
-#endif
-            {
-              /* We have been redirected, possibly to another host, or
-                 different path, or wherever.  Check whether we really
-                 want to follow it.  */
-              if (descend)
-                {
-#ifndef ENABLE_THREADS
-                  if (!descend_redirect_p (redirected, url_parsed, depth,
-#else
-                  if (!descend_redirect_p (thread_ctx[index].redirected,
-                                           thread_ctx[index].url_parsed, depth,
-#endif
-                                           start_url_parsed, blacklist, i))
-                    descend = false;
-                  else
-                    /* Make sure that the old pre-redirect form gets
-                       blacklisted. */
-                    string_set_add (blacklist, url);
-                }
-
-#ifndef ENABLE_THREADS
-              xfree (url);
-              url = redirected;
-#else
-              xfree (thread_ctx[index].url);
-              url = thread_ctx[index].redirected;
-#endif
+              char *error = url_error (url, url_err);
+              logprintf (LOG_NOTQUIET, "%s: %s.\n",url, error);
+              xfree (error);
+              inform_exit_status (URLERROR);
             }
           else
             {
-#ifndef ENABLE_THREADS
-              xfree (url);
-              url = xstrdup (url_parsed->url);
-#else
-              xfree (thread_ctx[index].url);
-              url = xstrdup (thread_ctx[index].url_parsed->url);
-#endif
+
+              status = retrieve_url (url_parsed, url, &file, &redirected, referer,
+                                     &dt, false, i, true);
+
+              if (html_allowed && file && status == RETROK
+                  && (dt & RETROKF) && (dt & TEXTHTML))
+                {
+                  descend = true;
+                  is_css = false;
+                }
+
+              /* a little different, css_allowed can override content type
+                 lots of web servers serve css with an incorrect content type
+              */
+              if (file && status == RETROK
+                  && (dt & RETROKF) &&
+                  ((dt & TEXTCSS) || css_allowed))
+                {
+                  descend = true;
+                  is_css = true;
+                }
+
+              if (redirected)
+                {
+                  /* We have been redirected, possibly to another host, or
+                     different path, or wherever.  Check whether we really
+                     want to follow it.  */
+                  if (descend)
+                    {
+                      reject_reason r = descend_redirect (redirected, url_parsed,
+                                        depth, start_url_parsed, blacklist, i);
+                      if (r == WG_RR_SUCCESS)
+                        {
+                          /* Make sure that the old pre-redirect form gets
+                             blacklisted. */
+                          blacklist_add (blacklist, url);
+                        }
+                      else
+                        {
+                          write_reject_log_reason (rejectedlog, r, url_parsed, start_url_parsed);
+                          descend = false;
+                        }
+                    }
+
+                  xfree (url);
+                  url = redirected;
+                }
+              else
+                {
+                  xfree (url);
+                  url = xstrdup (url_parsed->url);
+                }
+              url_free (url_parsed);
             }
-#ifndef ENABLE_THREADS
-          url_free(url_parsed);
-#else
-          url_free(thread_ctx[index].url_parsed);
-#endif
         }
 
       if (opt.spider)
@@ -569,12 +446,16 @@ retry:
 
               for (; child; child = child->next)
                 {
+                  reject_reason r;
+
                   if (child->ignore_when_downloading)
                     continue;
                   if (dash_p_leaf_HTML && !child->link_inline_p)
                     continue;
-                  if (download_child_p (child, url_parsed, depth, start_url_parsed,
-                                        blacklist, i))
+
+                  r = download_child (child, url_parsed, depth,
+                                      start_url_parsed, blacklist, i);
+                  if (r == WG_RR_SUCCESS)
                     {
                       ci = iri_new ();
                       set_uri_encoding (ci, i->content_encoding, false);
@@ -585,7 +466,11 @@ retry:
                       /* We blacklist the URL we have enqueued, because we
                          don't want to enqueue (and hence download) the
                          same URL twice.  */
-                      string_set_add (blacklist, child->url->url);
+                      blacklist_add (blacklist, child->url->url);
+                    }
+                  else
+                    {
+                      write_reject_log_reason (rejectedlog, r, child->url, url_parsed);
                     }
                 }
 
@@ -619,14 +504,16 @@ retry:
           logputs (LOG_VERBOSE, "\n");
           register_delete_file (file);
         }
-#ifndef ENABLE_THREADS
+
       xfree (url);
-      xfree_null (referer);
-      xfree_null (file);
+      xfree (referer);
+      xfree (file);
       iri_free (i);
-#endif
     }
 
+  if (rejectedlog)
+    fclose (rejectedlog);
+
   /* If anything is left of the queue due to a premature exit, free it
      now.  */
   {
@@ -639,7 +526,7 @@ retry:
       {
         iri_free (d6);
         xfree (d1);
-        xfree_null (d2);
+        xfree (d2);
       }
   }
   url_queue_delete (queue);
@@ -662,40 +549,43 @@ retry:
    by storing these URLs to BLACKLIST.  This may or may not help.  It
    will help if those URLs are encountered many times.  */
 
-static bool
-download_child_p (const struct urlpos *upos, struct url *parent, int depth,
+static reject_reason
+download_child (const struct urlpos *upos, struct url *parent, int depth,
                   struct url *start_url_parsed, struct hash_table *blacklist,
                   struct iri *iri)
 {
   struct url *u = upos->url;
   const char *url = u->url;
   bool u_scheme_like_http;
+  reject_reason reason = WG_RR_SUCCESS;
 
   DEBUGP (("Deciding whether to enqueue \"%s\".\n", url));
 
-  if (string_set_contains (blacklist, url))
+  if (blacklist_contains (blacklist, url))
     {
       if (opt.spider)
         {
           char *referrer = url_string (parent, URL_AUTH_HIDE_PASSWD);
-          DEBUGP (("download_child_p: parent->url is: %s\n", quote (parent->url)));
+          DEBUGP (("download_child: parent->url is: %s\n", quote (parent->url)));
           visited_url (url, referrer);
           xfree (referrer);
         }
       DEBUGP (("Already on the black list.\n"));
+      reason = WG_RR_BLACKLIST;
       goto out;
     }
 
   /* Several things to check for:
-     1. if scheme is not http, and we don't load it
-     2. check for relative links (if relative_only is set)
-     3. check for domain
-     4. check for no-parent
-     5. check for excludes && includes
-     6. check for suffix
-     7. check for same host (if spanhost is unset), with possible
+     1. if scheme is not https and https_only requested
+     2. if scheme is not http, and we don't load it
+     3. check for relative links (if relative_only is set)
+     4. check for domain
+     5. check for no-parent
+     6. check for excludes && includes
+     7. check for suffix
+     8. check for same host (if spanhost is unset), with possible
      gethostbyname baggage
-     8. check for robots.txt
+     9. check for robots.txt
 
      Addendum: If the URL is FTP, and it is to be loaded, only the
      domain and suffix settings are "stronger".
@@ -707,13 +597,23 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
      More time- and memory- consuming tests should be put later on
      the list.  */
 
+#ifdef HAVE_SSL
+  if (opt.https_only && u->scheme != SCHEME_HTTPS)
+    {
+      DEBUGP (("Not following non-HTTPS links.\n"));
+      reason = WG_RR_NOTHTTPS;
+      goto out;
+    }
+#endif
+
   /* Determine whether URL under consideration has a HTTP-like scheme. */
   u_scheme_like_http = schemes_are_similar_p (u->scheme, SCHEME_HTTP);
 
   /* 1. Schemes other than HTTP are normally not recursed into. */
-  if (!u_scheme_like_http && !(u->scheme == SCHEME_FTP && opt.follow_ftp))
+  if (!u_scheme_like_http && !((u->scheme == SCHEME_FTP || u->scheme == SCHEME_FTPS) && opt.follow_ftp))
     {
       DEBUGP (("Not following non-HTTP schemes.\n"));
+      reason = WG_RR_NONHTTP;
       goto out;
     }
 
@@ -723,6 +623,7 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
     if (opt.relative_only && !upos->link_relative_p)
       {
         DEBUGP (("It doesn't really look like a relative link.\n"));
+        reason = WG_RR_ABSOLUTE;
         goto out;
       }
 
@@ -731,6 +632,7 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
   if (!accept_domain (u))
     {
       DEBUGP (("The domain was not accepted.\n"));
+      reason = WG_RR_DOMAIN;
       goto out;
     }
 
@@ -750,6 +652,7 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
         {
           DEBUGP (("Going to \"%s\" would escape \"%s\" with no_parent on.\n",
                    u->dir, start_url_parsed->dir));
+          reason = WG_RR_PARENT;
           goto out;
         }
     }
@@ -762,12 +665,14 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
       if (!accdir (u->dir))
         {
           DEBUGP (("%s (%s) is excluded/not-included.\n", url, u->dir));
+          reason = WG_RR_LIST;
           goto out;
         }
     }
   if (!accept_url (url))
     {
       DEBUGP (("%s is excluded/not-included through regex.\n", url));
+      reason = WG_RR_REGEX;
       goto out;
     }
 
@@ -792,6 +697,7 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
         {
           DEBUGP (("%s (%s) does not match acc/rej rules.\n",
                    url, u->file));
+          reason = WG_RR_RULES;
           goto out;
         }
     }
@@ -802,6 +708,7 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
       {
         DEBUGP (("This is not the same hostname as the parent's (%s and %s).\n",
                  u->host, parent->host));
+        reason = WG_RR_SPANNEDHOST;
         goto out;
       }
 
@@ -843,36 +750,37 @@ download_child_p (const struct urlpos *upos, struct url *parent, int depth,
       if (!res_match_path (specs, u->path))
         {
           DEBUGP (("Not following %s because robots.txt forbids it.\n", url));
-          string_set_add (blacklist, url);
+          blacklist_add (blacklist, url);
+          reason = WG_RR_ROBOTS;
           goto out;
         }
     }
 
-  /* The URL has passed all the tests.  It can be placed in the
-     download queue. */
-  DEBUGP (("Decided to load it.\n"));
+  out:
 
-  return true;
+  if (reason == WG_RR_SUCCESS)
+    /* The URL has passed all the tests.  It can be placed in the
+       download queue. */
+    DEBUGP (("Decided to load it.\n"));
+  else
+    DEBUGP (("Decided NOT to load it.\n"));
 
- out:
-  DEBUGP (("Decided NOT to load it.\n"));
-
-  return false;
+  return reason;
 }
 
 /* This function determines whether we will consider downloading the
    children of a URL whose download resulted in a redirection,
    possibly to another host, etc.  It is needed very rarely, and thus
-   it is merely a simple-minded wrapper around download_child_p.  */
+   it is merely a simple-minded wrapper around download_child.  */
 
-static bool
-descend_redirect_p (const char *redirected, struct url *orig_parsed, int depth,
+static reject_reason
+descend_redirect (const char *redirected, struct url *orig_parsed, int depth,
                     struct url *start_url_parsed, struct hash_table *blacklist,
                     struct iri *iri)
 {
   struct url *new_parsed;
   struct urlpos *upos;
-  bool success;
+  reject_reason reason;
 
   assert (orig_parsed != NULL);
 
@@ -882,16 +790,104 @@ descend_redirect_p (const char *redirected, struct url *orig_parsed, int depth,
   upos = xnew0 (struct urlpos);
   upos->url = new_parsed;
 
-  success = download_child_p (upos, orig_parsed, depth,
+  reason = download_child (upos, orig_parsed, depth,
                               start_url_parsed, blacklist, iri);
 
+  if (reason == WG_RR_SUCCESS)
+    blacklist_add (blacklist, upos->url->url);
+  else
+    DEBUGP (("Redirection \"%s\" failed the test.\n", redirected));
+
   url_free (new_parsed);
   xfree (upos);
 
-  if (!success)
-    DEBUGP (("Redirection \"%s\" failed the test.\n", redirected));
+  return reason;
+}
 
-  return success;
+
+/* This function writes the rejected log header. */
+static void
+write_reject_log_header (FILE *f)
+{
+  if (!f)
+    return;
+
+  /* Note: Update this header when columns change in any way. */
+  fprintf (f, "REASON\t"
+    "U_URL\tU_SCHEME\tU_HOST\tU_PORT\tU_PATH\tU_PARAMS\tU_QUERY\tU_FRAGMENT\t"
+    "P_URL\tP_SCHEME\tP_HOST\tP_PORT\tP_PATH\tP_PARAMS\tP_QUERY\tP_FRAGMENT\n");
+}
+
+/* This function writes a URL to the reject log. Internal use only. */
+static void
+write_reject_log_url (FILE *fp, const struct url *url)
+{
+  const char *escaped_str;
+  const char *scheme_str;
+
+  if (!fp)
+    return;
+
+  escaped_str = url_escape (url->url);
+
+  switch (url->scheme)
+    {
+      case SCHEME_HTTP:  scheme_str = "SCHEME_HTTP";    break;
+#ifdef HAVE_SSL
+      case SCHEME_HTTPS: scheme_str = "SCHEME_HTTPS";   break;
+      case SCHEME_FTPS:  scheme_str = "SCHEME_FTPS";    break;
+#endif
+      case SCHEME_FTP:   scheme_str = "SCHEME_FTP";     break;
+      default:           scheme_str = "SCHEME_INVALID"; break;
+    }
+
+  fprintf (fp, "%s\t%s\t%s\t%i\t%s\t%s\t%s\t%s",
+    escaped_str,
+    scheme_str,
+    url->host,
+    url->port,
+    url->path,
+    url->params ? url->params : "",
+    url->query ? url->query : "",
+    url->fragment ? url->fragment : "");
+
+  xfree (escaped_str);
+}
+
+/* This function writes out information on why a URL was rejected and its
+   context from download_child such as the URL being rejected and it's
+   parent's URL. The format it uses is comma separated values but with tabs. */
+static void
+write_reject_log_reason (FILE *fp, reject_reason reason,
+                         const struct url *url, const struct url *parent)
+{
+  const char *reason_str;
+
+  if (!fp)
+    return;
+
+  switch (reason)
+    {
+      case WG_RR_SUCCESS:     reason_str = "SUCCESS";     break;
+      case WG_RR_BLACKLIST:   reason_str = "BLACKLIST";   break;
+      case WG_RR_NOTHTTPS:    reason_str = "NOTHTTPS";    break;
+      case WG_RR_NONHTTP:     reason_str = "NONHTTP";     break;
+      case WG_RR_ABSOLUTE:    reason_str = "ABSOLUTE";    break;
+      case WG_RR_DOMAIN:      reason_str = "DOMAIN";      break;
+      case WG_RR_PARENT:      reason_str = "PARENT";      break;
+      case WG_RR_LIST:        reason_str = "LIST";        break;
+      case WG_RR_REGEX:       reason_str = "REGEX";       break;
+      case WG_RR_RULES:       reason_str = "RULES";       break;
+      case WG_RR_SPANNEDHOST: reason_str = "SPANNEDHOST"; break;
+      case WG_RR_ROBOTS:      reason_str = "ROBOTS";      break;
+      default:                reason_str = "UNKNOWN";     break;
+    }
+
+  fprintf (fp, "%s\t", reason_str);
+  write_reject_log_url (fp, url);
+  fprintf (fp, "\t");
+  write_reject_log_url (fp, parent);
+  fprintf (fp, "\n");
 }
 
 /* vim:set sts=2 sw=2 cino+={s: */
