@@ -3,6 +3,7 @@ from sys import exit
 from test.http_test import HTTPTest
 from test.base_test import HTTP, HTTPS
 from misc.wget_file import WgetFile
+import os
 
 """
     This test ensures that Wget can download files from HTTPS Servers
@@ -17,7 +18,8 @@ A_File = WgetFile ("File1", File1)
 B_File = WgetFile ("File2", File2)
 C_File = WgetFile ("File3", File3)
 
-WGET_OPTIONS = "--no-check-certificate"
+CAFILE = os.path.abspath(os.path.join(os.getenv('srcdir', '.'), 'certs', 'ca-cert.pem'))
+WGET_OPTIONS = "--ca-certificate=" + CAFILE
 WGET_URLS = [["File1", "File2"]]
 
 Files = [[A_File, B_File]]
